﻿using KitchenRoot;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfKitchen
{
	public class TreeNode : XObjectNotifier
	{
		protected string		_title;
		protected bool			_isExclusive;		// radio-style option? / check-style option?
		protected bool?			_isSelected		= null;
		protected TreeNode		_parent;
		protected TreeNodeList	_children;

		public TreeNode()
		{

		}

		public TreeNode(TreeNode parent, string title, bool isSelected, bool isExclusive)
		{
			_parent			= parent;
			_title			= title;
			_isExclusive	= isExclusive;
			_isSelected		= isSelected;
		}


		public string Title
		{
			get {  return _title; }
			set
			{
				if( value == _title)
					return;

				_title = value;
				NotifyPropertyChanged(() => Title);
			}
		}

		public TreeNode Parent
		{
			get {  return _parent; }
			set
			{
				if( value == _parent)
					return;

				_parent = value;
				NotifyPropertyChanged(() => Parent);
				NotifyPropertyChanged(() => RootNode);
			}
		}

		public bool IsExclusive
		{
			get {  return _isExclusive; }
			set
			{
				if( value == _isExclusive)
					return;

				_isExclusive = value;
				NotifyPropertyChanged(() => IsExclusive);
				UpdateSelection();
			}
		}

		public bool HasSelection
		{
			get
			{
				if(_children == null)
					return false;

				return _children.HasSelection;
			}
		}


		public bool? IsSelected
		{
			get {  return _isSelected; }
			set
			{
				if( value == _isSelected)
					return;

				_isSelected = value;
				NotifyPropertyChanged(() => IsSelected);
				NotifyPropertyChanged(() => Parent.HasSelection);
				UpdateSelection();
			}
		}

		/// <summary>
		/// set the selection without updating siblings
		/// </summary>
		/// <param name="value"></param>
		protected void SetSelection(bool value, bool updateChildren = true)
		{
			if(value != _isSelected)
			{
				_isSelected = value;
				NotifyPropertyChanged(() => IsSelected);
			}

			if(updateChildren)
				SelectChildren(value);
		}


		public TreeNodeList Children
		{
			get {  return _children; }
			set
			{
				if( value == _children)
					return;

				_children = value;
				NotifyPropertyChanged(() => Children);
				NotifyPropertyChanged(() => FirstSelectedDescendant);
				NotifyPropertyChanged(() => HasSelectedDescendants);
				NotifyPropertyChanged(() => HasSelection);
			}
		}

		public TreeNode RootNode
		{
			get
			{
				TreeNode	root		= this;

				while (root.Parent != null)
				{
					root = root.Parent;
				}
				return root;
			}
		}

		public TreeNode FirstExclusiveParent
		{
			get
			{
				if (this._isExclusive)
					return this;

				if (_parent != null)
					return _parent.FirstExclusiveParent;

				return null;
			}
		}

		public TreeNode FirstExclusiveChild
		{
			get
			{
				if(_children == null)
					return null;

				return _children.FirstOrDefault(i => i.IsExclusive);
			}
		}

		public TreeNode FirstSelectedDescendant
		{
			get
			{
				if(_children == null)
					return null;
				return this._children.FirstSelectedDescendant;
			}
		}

		public bool HasSelectedDescendants
		{
			get {  return FirstSelectedDescendant != null; }
		}

		public void UpdateSelection()
		{
			if(! _isExclusive)
				return;

			if(_isSelected == null)
				return;

			if(_isSelected == true)
			{
				UnSelectSiblings();
			}

			SelectChildren(_isSelected.Value);
		}

		protected void UnSelectChildren()
		{
			if (_children == null)
				return;

			foreach (TreeNode node in _children)
			{
				node.SetSelection(value: false, updateChildren: true);
			}
		}

		protected void SelectChildren(bool selected)
		{
			if(! selected)
			{
				UnSelectChildren();
				return;
			}

			if(_children == null)
				return;

			TreeNode firstEx = FirstExclusiveChild;

			if(firstEx != null)
				firstEx.IsSelected = selected;

			foreach(TreeNode node in _children)
			{
				if(node.IsExclusive)
					continue;

				node.SetSelection(value: selected, updateChildren: true);
			}
		}

		protected void UnSelectSiblings()
		{
			//UnSelectHigestSiblings();

			if (_parent == null)
				return;

			foreach (TreeNode other in _parent.Children)
			{
				if (other == this)
					continue;

				TreeNode	firstEx	= this.FirstExclusiveParent;

				if(other.IsExclusive)
					other.SetSelection( false, true);
			}
		}
	}
}
